try:
    from Products.validation.interfaces.IValidator import IValidator
except ImportError:
    import sys, os
    sys.path.insert(0, os.path.join(os.path.dirname(__file__), os.pardir))
    from interfaces.IValidator import IValidator
    del sys, os

import re
from types import StringType

def ignoreRE(value, expression):
    ignore = re.compile(expression)
    return ignore.sub('', value)

class RegexValidator:
    __implements__ = IValidator

    def __init__(self, name, *args, **kw):
        self.name = name
        self.title = kw.get('title', name)
        self.description = kw.get('description', '')
        self.regex = []
        for r in args:
            self.regex.append(re.compile(r))

        self.ignore = kw.get('ignore', None)


    def __call__(self, value, *args, **kwargs):
        if type(value) != StringType:
            return ("Validation Failed(%s): %s of "
                    "type %s, expected 'string'" %
                    (self.name, value, type(value)))

        ignore = kwargs.get('ignore', None)
        if ignore:
            value = ignoreRE(value, ignore)
        elif self.ignore:
            value = ignoreRE(value, self.ignore)


        for r in self.regex:
            m = r.match(value)
            if not m:
                return ("Validation Failed(%s): on "
                        "%s" % (self.name, value))
        return 1
